#
# Example wscp commands for deploying and creating (installing)  
# an enterprise bean
#
# The script init.tcl must be loaded before running this script
# Substitute different values for the following variables, as needed:
# data -- the name of your datasource, if installing CMP entity beans
# serv -- the name of your application server
# cont -- the name of the container in your application server
# jar -- the full path name of the undeployed (EJB JAR) file 
#        containing the bean to be deployed;
#        the example JAR file used here is Card.jar
#
#	   This variable is then reset to the full pathname of
#        the deployed (EJS JAR) file
# dd -- the name of the serialized deployment descriptor for the bean
# bean -- the name of the enterprise bean
# attrs -- the attribute list for setting the DeploymentDescriptor and JarFile
#          attributes; add any other additional attributes you may need, for
#          example, set the DataSource attribute for CMP entity beans

set data "/DataSource:testDataSource/"

set serv "${NODE}ApplicationServer:AppServer2/"
# Create an ApplicationServer object
    set cmd {ApplicationServer create $serv}
    if {$VERBOSE} {puts [subst "# $cmd"]}
    eval $cmd


set cont "${serv}EJBContainer:AppServer2 EJB Container/"
# Create an EJBContainer object
    set attrs "{DataSource $data}"
    set cmd {EJBContainer create $cont -attribute $attrs}
    if {$VERBOSE} {puts [subst "# $cmd"]}
    eval $cmd

# Deploy an enterprise bean (example JAR file is Card.jar)
# The option -WLMize is used to prepare the enterprise bean for workload management

set jar "${WAS_HOME}${FILE_SEPARATOR}deployableEJBs${FILE_SEPARATOR}Card.jar"
set dd "$jar/com.transarc.systest.ejs.debitCard/card1.ser"

set cmd {EnterpriseBean deploy $dd -node $NODE -trace -WLMize}
if {$VERBOSE} {puts [subst "# $cmd"]}
    eval $cmd

set bean "${cont}EnterpriseBean:myCardBean/"

# Create (install) an enterprise bean

set jar "${WAS_HOME}${FILE_SEPARATOR}deployedEJBs${FILE_SEPARATOR}DeployedCard.jar" 
set attrs "{DeploymentDescriptor com.transarc.systest.ejs.debitCard.Card@$jar}{JarFile $jar}{DataSource $data}"
    
set cmd {EnterpriseBean create $bean -attribute $attrs}
if {$VERBOSE} {puts [subst "# $cmd"]}
    eval $cmd

set cmd {EnterpriseBean start $bean}
if {$VERBOSE} {puts [subst "# $cmd"]}
    eval $cmd
   

