// 5648-C84, 5648-C83, (C) Copyright IBM Corporation, 1997, 2000
// All rights reserved. Licensed Materials Property of IBM
// Note to US Government users: Documentation related to restricted rights
// Use, duplication or disclosure is subject to restrictions set forth in GSA ADP Schedule with IBM Corp.
// This page may contain other proprietary notices and copyright information, the terms of which must be observed and followed.
//
// This program may be used, executed, copied, modified and distributed
// without royalty for the purpose of developing, using,
// marketing, or distributing.
//
// @(#) 1.4 src/ejb/java/com/ibm/ejs/doc/account/AccountBean.java, ejbdemo, boss 5/13/99 14:22:17 [5/25/99 15:37:32]

package com.ibm.ejs.doc.account;

/**
 * The bean class of the Account entity bean.
 **/
import java.rmi.RemoteException;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.ListResourceBundle;
import javax.ejb.*;
import java.lang.*;

public class AccountBean implements EntityBean {
    private EntityContext entityContext = null;
    
    private ListResourceBundle bundle = (ListResourceBundle)
	ResourceBundle.getBundle("com.ibm.ejs.doc.account.AccountResourceBundle");
    
    public long accountId = 0;
    public int type = 1;
    public float balance = 0.0f;
    /**
     * Adds the specified amount to the account balance and returns the new balance.
     * @return float The new balance in the account after the method executes.
     * @param amount float The amount to add to the account balance.
     */
    public float add(float amount) {
	balance += amount;
	return balance;
    }
    /**
     * Notifies the bean instance that it is being activated.
     * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
     */
    public void ejbActivate() throws RemoteException { }
    /**
     * Notifies the bean instance that it is being created and invokes the other ejbCreate
     * method with default values for the account type (1 for savings) and the initial balance
     * (0).
     * @param key com.ibm.ejs.doc.account.AccountKey The AccountKey object that encapsulates the
     * account Id used to create the specified account.
     */
    public void ejbCreate(AccountKey key) { 
	ejbCreate(key, 1, 0.0f); 
    }
    /**
     * Notifies the bean instance that it is being created and sets the values of the
     * persistent variables required by the container to create the bean instance.
     * @param key com.ibm.ejs.doc.account.AccountKey The AccountKey object that encapsulates the
     * account Id used to create the specified account.
     * @param type int The type of the account to be created: 1 for savings, 2 for checking.
     * @param initialBalance float The initial balance contained in the account.
     */
    public void ejbCreate(AccountKey key, int type, float initialBalance) {
	accountId = key.accountId;
	this.type = type;
	balance = initialBalance;
	
    }
    /**
     * Notifies the bean instance that the data in the data source is overwriting the
     * the values of the persistent data variables.
     * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
     */
    public void ejbLoad () throws RemoteException { }
    /**
     * Notifies the bean instance that it is being passivated.
     * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
     */
    public void ejbPassivate() throws RemoteException { }
    /**
     * Notifies the bean instance that it has been created.
     * @param key com.ibm.ejs.doc.account.AccountKey The AccountKey object that encapsulates the
     * account Id used to create the specified account.
     * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
     */
    public void ejbPostCreate(AccountKey key) throws RemoteException { 
	ejbPostCreate(key, 1, 0.0f); 
    }
    /**
     * Notifies the bean instance that it has been created.
     * @param key com.ibm.ejs.doc.account.AccountKey The AccountKey object that encapsulates the
     * account Id used to create the specified account.
     * @param type int The type of the account created: 1 for savings, 2 for checking.
     * @param initialBalance float The initial balance contained in the account.
     */
    public void ejbPostCreate(AccountKey key, int type, float initialBalance) { }
    /**
     * Notifies the bean instance that it is being removed.
     * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
     */
    public void ejbRemove() throws RemoteException { }
    /**
     * Notifies the bean instance that the values of the persistent data variables are
     * being written to the data source, overwriting the values in the data source.
     * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
     */
    public void ejbStore () throws RemoteException { }
    /**
     * Returns the balance for the account.
     * @return float The balance in the account.
     */
    public float getBalance() { 
	return balance; 
    }
    /**
     * Sets the account balance to the specified amount.
     * @param amount float The new account balance.
     */
    public void setBalance(float amount) { 
	balance = amount; 
    }
    /**
     * Sets the entity context for the bean instance.
     * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
     */
    public void setEntityContext(EntityContext ctx) throws RemoteException { 
	entityContext = ctx; 
    }
    /**
     * Subtracts the specified amount from the account balance and returns the new balance.
     * @return float The new balance in the account after the method executes.
     * @param amount float The amount to subtract from the account balance.
     * @exception com.ibm.ejs.doc.account.InsufficientFundsException The account contained 
     * less than the amount requested for subtraction.
     */
    public float subtract(float amount) throws InsufficientFundsException {
	if(balance < amount) {
	    throw new InsufficientFundsException(bundle.getString("insufficientFunds"));
	}	
	balance -= amount;
	return balance;
    }
    /**
     * Unsets the entity context for the bean instance.
     * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
     */
    public void unsetEntityContext() throws RemoteException { 
	entityContext = null; 
    }
}






