// 5648-C84, 5648-C83, (C) Copyright IBM Corporation, 1997, 2000
// All rights reserved. Licensed Materials Property of IBM
// Note to US Government users: Documentation related to restricted rights
// Use, duplication or disclosure is subject to restrictions set forth in GSA ADP Schedule with IBM Corp.
// This page may contain other proprietary notices and copyright information, the terms of which must be observed and followed.
//
// This program may be used, executed, copied, modified and distributed
// without royalty for the purpose of developing, using,
// marketing, or distributing.
//
package WebSphereSamples.AccountAndTransfer;

// General EJB.
import java.rmi.RemoteException;
import javax.ejb.DuplicateKeyException;

// EJB specific for this servlet.
import com.ibm.ejs.doc.account.AccountHome;
import com.ibm.ejs.doc.account.AccountKey;
import com.ibm.ejs.doc.account.Account;

// Servlet related.
import javax.servlet.*;
import javax.servlet.http.*;

// JNDI (naming).
import javax.naming.*;  // for Context, InitialContext, NamingException

// Miscellaneous:
import java.util.*;
import java.io.*;

// StudioPervasiveServlet extends HttpServlet
import com.ibm.webtools.runtime.StudioPervasiveServlet;

/**
 * Utility servlet to create an Account entity bean.
 **/
public class CreateAccountJSP extends com.ibm.webtools.runtime.StudioPervasiveServlet {

   // Variables for finding the home
   private String nameService = null;
   private String accountName = null;    
   private String providerURL = null;
   private ResourceBundle bundle = 
   ResourceBundle.getBundle("WebSphereSamples.AccountAndTransfer.ClientResourceBundle");

   // Strings for HTML output - US English defaults shown.
   static String createdaccount  = "Created account:";
   static String createdtype     = "Type:";
   static String createdbalance  = "Balance:";
   static String failureexists   = "Create failure - account already exists:";
   static String failureinternal = "Internal failure - could not create account:";
   static String failureaccount  = "Invalid account format:";
   static String failurebalance  = "Invalid balance format:";
   static String initError       = "Error initializing Servlet.";

   // EJB access.
   private AccountHome accountHome = null;

   /**
    * Read in internationalization strings when servlet is loaded. <p>
    * Create AccountHome when servlet is loaded. <p>
    * @param config Configuration information passed by the servlet engine.
    * @exception javax.servlet.ServletException This exception is thrown to indicate a servlet problem.
    */
   public void init(ServletConfig config) throws ServletException
   {
      super.init(config);
      // Get national language text for HTML response.
      try {
         // Get NLS strings from an external resource bundle identified by 
         // "WebSphereSamples.AccountAndTransfer.ClientResourceBundle", to override 
         // US English defaults.

         createdaccount  = bundle.getString("createdaccount"); 
         createdtype     = bundle.getString("createdtype");    
         createdbalance  = bundle.getString("createdbalance"); 
         failureexists   = bundle.getString("failureexists");  
         failureinternal = bundle.getString("failureinternal");
         failureaccount  = bundle.getString("failureaccount"); 
         failurebalance  = bundle.getString("failurebalance");
         initError       = bundle.getString("initError");

         //Get values for the naming factory and home name.
         nameService     = bundle.getString("nameService");
         accountName     = bundle.getString("accountName");
         providerURL     = bundle.getString("providerURL");

      } catch (Exception e) {
         System.out.println("init resource bundle exception: " + e.getMessage());
         throw new ServletException(e.getMessage());
      }

      // Get home object for Account EJB access.
      Hashtable env = new Hashtable();

      env.put(Context.INITIAL_CONTEXT_FACTORY,
              nameService);
      try {
         // Create the initial context.
         Context ctx = new InitialContext(env);
         // Get the home object.
         Object homeObject = ctx.lookup(accountName);

         accountHome = (AccountHome) javax.rmi.PortableRemoteObject.narrow(homeObject, AccountHome.class );

      }
      // Determine cause of failure.
      catch (NamingException e) {
         System.out.println(bundle.getString("failureinit") + e.getMessage() );
         e.printStackTrace();
         throw new ServletException(e.getMessage());
      } catch (Exception e) {
         System.out.println(bundle.getString("failuregenexp") + e.getMessage() );
         e.printStackTrace();
         throw new ServletException(e.getMessage());
      }
   }

   /**
    * Respond to user request to create an Account entity bean. <p>
    * @param req Used to get information about the user request - an account number, etc.
    * @param res Used to pass response back to the user - an output stream for the returned HTML, etc.
    * @exception javax.servlet.ServletException This exception is thrown to indicate a servlet problem.
    * @exception java.io.IOException Signals that an I/O exception of some sort has occurred.
    */
   public void performTask(HttpServletRequest req, HttpServletResponse res)
   {
      // --- Read and validate user input, initialize. ---
      // Error flags.      
      boolean accountFlag   = true;
      boolean balanceFlag   = true;
      boolean inputFlag     = false;
      boolean createFlag    = true;
      boolean duplicateFlag = false;

      // Datatypes used to create new account.
      AccountKey key;
      int        typeAcct       = 0;
      String     typeString     = "0";
      float      initialBalance = 0;

      // Read input parameters from HTML form.
      String[] accountArray = req.getParameterValues("account");
      String[] typeArray = req.getParameterValues("type");
      String[] balanceArray = req.getParameterValues("balance");

      // Convert input parameters to needed datatypes for new account EJB.
      // (account)
      long accountLong = 0;
      try {
         accountLong = Long.parseLong(accountArray[0]);
      } catch (Exception e) {
         accountFlag = false;
      }
      key = new AccountKey(accountLong);

      // (type)
      if (typeArray[0].equals("1")) {
         typeAcct   = 1;           // Savings account.
         typeString = "savings";
      } else if (typeArray[0].equals("2")) {
         typeAcct   = 2;           // Checking account
         typeString = "checking";
      }

      // (balance)
      try {
         initialBalance = (Float.valueOf(balanceArray[0])).floatValue();
      } catch (Exception e) {
         balanceFlag = false;
      }

      // --- If input parameters are good, try to create account EJB. ---
      if (accountFlag && balanceFlag) {
         inputFlag = true;
         try {
            // Create the EJB.
            Account account = accountHome.create(key, typeAcct, initialBalance);
         }
         // Determine cause of failure.
         catch (RemoteException e) {
            createFlag = false;
            System.out.println(bundle.getString("failureremote") + e.getMessage() );
            e.printStackTrace();
         } catch (DuplicateKeyException e) {
            createFlag    = false;
            duplicateFlag = true;
            System.out.println(bundle.getString("failuredupkey") + e.getMessage() );
            e.printStackTrace();
         } catch (Exception e) {
            createFlag = false;
            System.out.println(bundle.getString("failuregenexp") + e.getMessage() );
            e.printStackTrace();
         }
      }

      // --- Prepare message to accompany response. ---
      String messageLine  = "";
      if (inputFlag) {
         // If you are here, the client input is good.
         if (createFlag) {
            // New account EJB was created.
            messageLine =  createdaccount + " " + accountArray[0] + ", " + 
                           createdtype + " " + typeString + ", " + 
                           createdbalance + " " + balanceArray[0]; 
         } else if (duplicateFlag) {
            // Account with same key already exists.
            messageLine = failureexists + " " + accountArray[0];
         } else {
            // Other reason for failure.
            messageLine = failureinternal + " " + accountArray[0]; 
         }
      } else {
         // If you are here, something was wrong with the client input.
         String separator = "";
         if (!accountFlag) {
            messageLine = failureaccount + " " + accountArray[0];
            separator = ", ";
         }
         if (!balanceFlag) {
            messageLine = messageLine + separator + 
                          failurebalance + " " + balanceArray[0];
         }
      }

      JSPBean jspbean = new JSPBean();
      jspbean.setMessageLine(messageLine);
      javax.servlet.http.HttpSession session = req.getSession(true);
      session.putValue("JSPBean", jspbean);
      String pageName = getPageNameFromRequest(req);
      System.out.println("CreateJSP: pageName: " + pageName);
      callPage(pageName, req, res);

   }
}
