// 5648-C84, 5648-C83, (C) Copyright IBM Corporation, 1997, 2000
// All rights reserved. Licensed Materials Property of IBM
// Note to US Government users: Documentation related to restricted rights
// Use, duplication or disclosure is subject to restrictions set forth in GSA ADP Schedule with IBM Corp.
// This page may contain other proprietary notices and copyright information, the terms of which must be observed and followed.
//
// This program may be used, executed, copied, modified and distributed
// without royalty for the purpose of developing, using,
// marketing, or distributing.

package WebSphereSamples.AccountAndTransfer;

// general enterprise beans
import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.ejb.FinderException;

// Enterprise bean specific for this application
import com.ibm.ejs.doc.transfer.TransferHome;
import com.ibm.ejs.doc.transfer.Transfer;
import com.ibm.ejs.doc.account.InsufficientFundsException;

// JNDI (naming)
import javax.naming.*;  // for Context, InitialContext, NamingException

// Miscellaneous 
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

public class TransferApplication extends JFrame implements ActionListener {
   // Access resource bundle - use bundle.getString() get strings in appropriate national
   // language for visual components (just below), for messages, and for JNDI lookup and 
   // and for access to enterprise beans.  This promotes portability.
   private ResourceBundle bundle = 
   ResourceBundle.getBundle("WebSphereSamples.AccountAndTransfer.ClientResourceBundle");

   // Visual components on user GUI
   private JLabel amountLabel      = new JLabel(bundle.getString("amount"), SwingConstants.CENTER);
   private JLabel fromAccountLabel = new JLabel(bundle.getString("from"), SwingConstants.CENTER);
   private JLabel toAccountLabel   = new JLabel(bundle.getString("to"), SwingConstants.CENTER);
   private JLabel fromBalanceLabel = new JLabel(bundle.getString("transferBalance"), SwingConstants.CENTER);
   private JLabel toBalanceLabel   = new JLabel(bundle.getString("transferBalance"), SwingConstants.CENTER);
   private JLabel statusLabel      = new JLabel(bundle.getString("status"));
   private JTextField amountField      = new JTextField();
   private JTextField fromAccountField = new JTextField();
   private JTextField toAccountField   = new JTextField();
   private JTextField fromBalanceField = new JTextField();
   private JTextField toBalanceField   = new JTextField();
   private JTextField statusField      = new JTextField(50);
   private JButton transferButton       = new JButton(bundle.getString("transfer"));
   private JButton clearButton          = new JButton(bundle.getString("clear"));
   private JButton getFromBalanceButton = new JButton(bundle.getString("get"));
   private JButton getToBalanceButton   = new JButton(bundle.getString("get"));

   // Enterprise bean related
   private Transfer transfer = null;

/**
 * The main entrypoint to the TransferApplication when it is run as a Java application.
 * @param args java.lang.String[] The arguments to the Transfer application. There are none.
 */
   public static void main(String[] args) {
      new TransferApplication();
   }

/**
 * Default constructor for creating the TransferApplication GUI and getting
 * initial enterprise bean access.
 */
   public TransferApplication() {
      // --- Create GUI ---
      setTitle(bundle.getString("frameTitle"));
      Container c = getContentPane();
      c.setLayout(new BorderLayout(10, 10));
      c.setBackground(Color.lightGray);
      setResizable(false);

      // Central area of GUI
      JPanel p1 = new JPanel(new GridLayout(5, 3, 10, 5));
      c.add(p1, "North");
      // first row
      p1.add(amountLabel);
      p1.add(fromAccountLabel);
      p1.add(toAccountLabel);
      // second row
      p1.add(amountField);     
      p1.add(fromAccountField);
      p1.add(toAccountField); 
      amountField.setEnabled(false);   
      fromAccountField.setEnabled(false);
      toAccountField.setEnabled(false);
      // third row
      p1.add(new JLabel(" "));
      p1.add(fromBalanceLabel);
      p1.add(toBalanceLabel);
      // forth row
      p1.add(transferButton);
      p1.add(fromBalanceField);
      p1.add(toBalanceField);
      transferButton.setEnabled(false);
      fromBalanceField.setEditable(false);
      toBalanceField.setEditable(false);
      // fifth row
      p1.add(clearButton);      
      p1.add(getFromBalanceButton);
      p1.add(getToBalanceButton);
      clearButton.setEnabled(false);
      getFromBalanceButton.setEnabled(false);
      getToBalanceButton.setEnabled(false);

      // Status line at bottom of GUI
      JPanel p2 = new JPanel(new BorderLayout());
      p2.add(statusLabel, "North");
      JPanel p3 = new JPanel(new BorderLayout());
      p3.add(statusField, "North");
      statusField.setEditable(false);
      c.add(p2, "West");
      c.add(p3, "Center");

      // Set up listeners
      transferButton.addActionListener(this);      
      clearButton.addActionListener(this);         
      getFromBalanceButton.addActionListener(this);
      getToBalanceButton.addActionListener(this);  
      // Use of WindowAdapter allows implementation of only
      // needed methods of WindowListener interface.
      addWindowListener(new WindowAdapter(){
                        public void windowClosing(WindowEvent e){
                        if (transfer != null){
                          try {
                             transfer.remove();
                          }
                          catch (Exception ex) {
                             System.out.println(ex.getMessage());
                          }
                        }
                        System.exit(0);}}
                       );

      // Show GUI, with message that enterprise bean inits are starting.
      pack();
      show();
      statusField.setText(bundle.getString("startinit"));
      // --- End of GUI creation. ---

      // --- Set up access to the Transfer enterprise bean. ---
      boolean initSuccess = false;
      try {
         // Create environment for lookup - note use of resource bundle.
         Properties env = System.getProperties();
         env.put(Context.PROVIDER_URL, bundle.getString("providerURL"));                                    
         env.put(Context.INITIAL_CONTEXT_FACTORY, bundle.getString("nameService"));
         env.put("org.omg.CORBA.ORBClass", "com.ibm.CORBA.iiop.ORB");
         // Create the initial context.
         Context ctx = new InitialContext(env);
         // Get the home object - note use of resource bundle.
         Object homeObject = ctx.lookup(bundle.getString("transferName"));
         // Get the TransferHome object
         TransferHome transferHome = (TransferHome)javax.rmi.PortableRemoteObject.narrow(homeObject, TransferHome.class);
         // Finally, get the transfer session enterprise bean
         transfer = transferHome.create();
         initSuccess = true;
      }
      catch (NamingException e) {
         System.out.println("NamingException: " + e);
         statusField.setText(e.getMessage());
      }
      catch (CreateException e) {
         System.out.println("CreateException: " + e);
         statusField.setText(e.getMessage());
      }
      catch (RemoteException e) {
         System.out.println("RemoteException: " + e);
         statusField.setText(e.getMessage());
      }
      catch (Exception e) {
         System.out.println("Exception: " + e);
         statusField.setText(e.getMessage());
      }
      if (initSuccess) {
         statusField.setText(bundle.getString("endinit"));
         amountField.setEnabled(true);   
         fromAccountField.setEnabled(true);
         toAccountField.setEnabled(true);
         clearButton.setEnabled(true);
         getFromBalanceButton.setEnabled(true);
         getToBalanceButton.setEnabled(true);
         transferButton.setEnabled(true);
      }
      // --- End of setup for access to enterprise bean. ---
   }

/**
 * Method to handle events for the ActionListener interface.
 * @param e java.awt.event.ActionEvent An ActionEvent for the TransferApplication. There is an
 * action event (button press) for each of the buttons (Clear, Transfer, and Get Balance).
 */
   public void actionPerformed(ActionEvent e) {
      if (e.getSource() == transferButton) {
         doTransfer();
      }
      if (e.getSource() == clearButton) {
         amountField.setText("");     
         fromAccountField.setText("");
         toAccountField.setText("");  
         fromBalanceField.setText("");
         toBalanceField.setText("");  
         statusField.setText("");     
      }
      if (e.getSource() == getFromBalanceButton) {
         getBalanceFromAccount();
      }

      if (e.getSource() == getToBalanceButton) {
         getBalanceToAccount();
      }
   }

/**
 * getBalanceFromAccount:  (getFromBalanceButton.action.actionPerformed(java.awt.event.ActionEvent) 
 * --> transfer.getBalance())
 * @return float
 */
   private float getBalanceFromAccount() {
      float balance = 0;
      statusField.setText("");
      try {
         balance = transfer.getBalance(Long.parseLong(fromAccountField.getText()));
         fromBalanceField.setText("" + balance);
      }
      catch (NumberFormatException e) {
         statusField.setText(bundle.getString("failurebadid"));
      }
      catch (RemoteException e) {
         fromBalanceField.setText("0");
         statusField.setText(bundle.getString("failureremote"));
      }
      catch (Throwable e) {
         statusField.setText(e.getMessage());
      }
      return balance;
   }

/**
 * getBalanceToAccount:  (getToBalanceButton.action.actionPerformed(java.awt.event.ActionEvent) 
 * --> transfer.getBalance())
 * @return float
 */
   private float getBalanceToAccount() {
      float balance = 0;
      statusField.setText("");
      try {
         balance = transfer.getBalance(Long.parseLong(toAccountField.getText()));
         toBalanceField.setText("" + balance);
      }
      catch (NumberFormatException e) {
         statusField.setText(bundle.getString("failurebadid"));
      }
      catch (RemoteException e) {
         toBalanceField.setText("0");
         statusField.setText(bundle.getString("failureremote"));
      }
      catch (Throwable e) {
         statusField.setText(e.getMessage());
      }
      return balance;
   }

/**
 * doTransfer:  (transferButton.action.actionPerformed(java.awt.event.ActionEvent) 
 * --> transfer.transferFunds())
 */
   private void doTransfer() {
      try {
         statusField.setText("");
         transfer.transferFunds(Long.parseLong(fromAccountField.getText()), 
                                Long.parseLong(toAccountField.getText()), 
                                new Float(amountField.getText()).floatValue());
         getBalanceFromAccount();
         getBalanceToAccount();
         statusField.setText(bundle.getString("successtransfer"));
      }
      catch (RemoteException e) {
         statusField.setText(bundle.getString("failureremote"));
      }
      catch (NumberFormatException e) {
         statusField.setText(bundle.getString("failurebadnum"));
      }
      catch (InsufficientFundsException e) {
         statusField.setText(bundle.getString("failureinsufficient"));
      }
      catch (Throwable e) {
         statusField.setText(e.getMessage());
      }
   }
}

