// 5648-C84, 5648-C83, (C) Copyright IBM Corporation, 1997, 2000
// All rights reserved. Licensed Materials Property of IBM
// Note to US Government users: Documentation related to restricted rights
// Use, duplication or disclosure is subject to restrictions set forth in GSA ADP Schedule with IBM Corp.
// This page may contain other proprietary notices and copyright information, the terms of which must be observed and followed.
//
// This program may be used, executed, copied, modified and distributed
// without royalty for the purpose of developing, using,
// marketing, or distributing.
//
// @(#) 1.3 src/ejb/java/com/ibm/ejs/doc/client/TransferFunds.java, ejbdemo, boss 5/19/99 16:35:19 [5/25/99 15:39:00]
package WebSphereSamples.AccountAndTransfer;

// General EJB.
import java.rmi.RemoteException;
import javax.ejb.FinderException;

// EJB specific for this servlet.
import com.ibm.ejs.doc.transfer.TransferHome;
import com.ibm.ejs.doc.transfer.Transfer;
import com.ibm.ejs.doc.account.InsufficientFundsException;
//WAS 2.0 uses bean HomeHelpers...
//import com.ibm.ejs.doc.transfer.TransferHomeHelper;

// Servlet related.
import javax.servlet.*;
import javax.servlet.http.*;

// JNDI (naming).
import javax.naming.*;  // for Context, InitialContext, NamingException

// Miscellaneous:
import java.util.*;
import java.io.*;

/**
 * Demonstration servlet showing a session EJB accessing an entity EJB
 * to get account balances or to transfer funds between accounts.
 **/
public class TransferFunds extends HttpServlet {

    // Variables
    private static final String INITIAL_NAMING_FACTORY_SYSPROP = "java.naming.factory.initial";
    private static final String HOME_NAME_SYSPROP   = "JNDIName";

    private ResourceBundle bundle =  ResourceBundle.getBundle("WebSphereSamples.AccountAndTransfer.ClientResourceBundle");

    // Strings for HTML output - US English defaults shown.
    static String transferTitle       = "Transfer Funds between Account EJBs";
    static String amount              = "Amount";
    static String from                = "From Account";
    static String to                  = "To Account";
    static String transferBalance     = "Balance";
    static String transfer            = "Transfer";
    static String clear               = "Clear";
    static String get                 = "Get Balance";
    static String successtransfer     = "Transfer successful";
    static String successbalance      = "Balance query successful";
    static String failureinsufficient = "Insufficient funds - no transfer";
    static String failurerequest      = "Internal failure - could not do request";
    static String failureinput        = "Bad input - could not do request";
    static String failureinternal     = "Internal failure - could not create account:";
    static String failureamount       = "Invalid amount format:";
    static String failureaccount      = "Invalid account format:";
    static String failurefind         = "One (or both) accounts not found";
    static String failuregenexp       = "Exception:";

    // EJB access.
    private TransferHome transferHome = null;

    /**
     * Read in internationalization strings when servlet is loaded. <p>
     * Create TransferHome when servlet is first loaded. <p>
     * @param config Configuration information passed by the servlet engine.
     * @exception javax.servlet.ServletException This exception is thrown to indicate a servlet problem.
     */
    public void init(ServletConfig config) throws ServletException {
	super.init(config);
	// Get national language text for HTML response.
	try {
	    // Get NLS strings from an external resource bundle identified by
	    // "WebSphereSamples.AccountAndTransfer.ClientResourceBundle", to override
	    // US English defaults.

	    transferTitle       = bundle.getString("transferTitle");
	    amount              = bundle.getString("amount");
	    from                = bundle.getString("from");
	    to                  = bundle.getString("to");
	    transferBalance     = bundle.getString("transferBalance");
	    transfer            = bundle.getString("transfer");
	    clear               = bundle.getString("clear");
	    get                 = bundle.getString("get");
	    successtransfer     = bundle.getString("successtransfer");
	    successbalance      = bundle.getString("successbalance");
	    failureinsufficient = bundle.getString("failureinsufficient");
	    failurerequest      = bundle.getString("failurerequest");
	    failureinput        = bundle.getString("failureinput");
	    failureamount       = bundle.getString("failureamount");
	    failureaccount      = bundle.getString("failureaccount");
	    failurefind         = bundle.getString("failurefind");
	}
	catch (Exception e) {
	    System.out.println("init resource bundle exception: " + e.getMessage());
	    e.printStackTrace();
	    throw new ServletException(e.getMessage());
	}
	
	
	// Get home object for Transfer EJB access.
	Hashtable env = new Hashtable();
	
	try {

	    env.put(Context.INITIAL_CONTEXT_FACTORY,
		    bundle.getString("nameService"));

	    // Create the initial context.
	    Context ctx = new InitialContext(env);


	    // Get the home object.
	    Object homeObject = ctx.lookup(bundle.getString("transferName"));

	    // Get the TransferHome object.
	    transferHome = (TransferHome)javax.rmi.PortableRemoteObject.narrow(homeObject, TransferHome.class);

	}
	// Determine approximate cause of failure.
	catch (NamingException e) {
	    System.out.println("init NamingException: " + e.getMessage() );
	    e.printStackTrace();
	    throw new ServletException(e.getMessage());
 	}
	catch (Exception e) {
	    System.out.println("init Exception: "+ e.getMessage() );
	    e.printStackTrace();
	    throw new ServletException(e.getMessage());
 	}
    }

    /**
     * Respond to user request to transfer funds between accounts, or get an account balance. <p>
     * @param req Used to get information about the user request - an account number, etc.
     * @param res Used to pass response back to the user - an output stream for the returned HTML, etc.
     * @exception javax.servlet.ServletException This exception is thrown to indicate a servlet problem.
     * @exception java.io.IOException Signals that an I/O exception of some sort has occurred.
     */
    public void doGet(HttpServletRequest req, HttpServletResponse res)
	 throws ServletException, IOException
    {
	// --- Read and validate user input, initialize. ---
	// Error flags.
	boolean fromAccountFlag  = true;
	boolean toAccountFlag    = true;
	boolean amountFlag       = true;
	boolean inputFlag        = false;
	boolean transferFlag     = false;
	boolean insufficientFlag = false;
	boolean finderFlag       = false;
	
	// Datatypes used in Transfer object methods.
	float amountFloat  = 0;
	long  fromKey      = 0;
	long  toKey        = 0;
	
	// Used in creating return HTML.
	String fromBalance  = "***";
	String toBalance    = "***";
	String messageLine1 = "";
	String messageLine2 = "";
	String fromAccount  = "";
	String toAccount    = "";
	
	// Was transfer, fromgetbalance, or togetbalance button selected?
	String[] transferArray = req.getParameterValues("transfer");
	String[] fromArray     = req.getParameterValues("fromgetbalance");
	String[] toArray       = req.getParameterValues("togetbalance");
	int request = 0;
	
	if (transferArray != null && transferArray.length > 0)
	    request = 1;    // transfer between accounts.
	else if (fromArray != null && fromArray.length > 0)
	    request = 2;    // get a balance for "from" account.
	else if (toArray != null && toArray.length > 0)
	    request = 3;    // get a balance for "to" account.
	
	// Get user keyed in parameters...  some may be null.
	String[] fromAccountArray = req.getParameterValues("fromaccount");
	String[] toAccountArray   = req.getParameterValues("toaccount");
	String[] amountArray      = req.getParameterValues("amount");
	
	// Validate the user input.
	if (request == 1) {      // need all three user inputs
	    // from account
	    try {
		fromKey = Long.parseLong(fromAccountArray[0]);
		fromAccount = "" + fromKey;
	    }
	    catch (Exception e) {
		fromAccountFlag = false;
	    }
	    // to account
	    try {
		toKey = Long.parseLong(toAccountArray[0]);
		toAccount = "" + toKey;
	    }
	    catch (Exception e) {
		toAccountFlag = false;
	    }
	    // amount to transfer
	    try {
		amountFloat = (Float.valueOf(amountArray[0])).floatValue();
	    }
	    catch (Exception e) {
		amountFlag = false;
	    }
	}
	else if (request == 2) { // just need account for balance query
	    // fromaccount balance
	    try {
		fromKey = Long.parseLong(fromAccountArray[0]);
		fromAccount = "" + fromKey;
	    }
	    catch (Exception e) {
		fromAccountFlag = false;
	    }
	}
	else if (request == 3) { // just need account for balance query
	    // toaccount balance
	    try {
		toKey = Long.parseLong(toAccountArray[0]);
		toAccount = "" + toKey;
	    }
	    catch (Exception e) {
		fromAccountFlag = false;
		fromAccountArray[0] = toAccountArray[0];
	    }
	}
	
	// --- If input parameters are good, try to transfer or get balance. ---
	if (amountFlag && fromAccountFlag && toAccountFlag) {
	    inputFlag = true;  // Client input is good if we get here.
	    Transfer transfer = null;
	
	    try {
		// Create a Transfer object.
		transfer = transferHome.create();
		// Based on user request type, invoke appropriate transfer methods.
		if (request == 1) {
		    transfer.transferFunds(fromKey, toKey, amountFloat);
		    fromBalance = "" + transfer.getBalance(fromKey);
		    toBalance   = "" + transfer.getBalance(toKey);
		    transferFlag = true;
		}
		else if (request == 2) {
		    fromBalance = "" + transfer.getBalance(fromKey);
		    transferFlag = true;
		}
		else if (request == 3) {
		    toBalance = "" + transfer.getBalance(toKey);
		    transferFlag = true;
		}
	    }
	    // Determine approximate cause of failure.
	    catch (InsufficientFundsException e) {
		insufficientFlag = true;
		System.out.println("InsufficientFundsException: " + e.getMessage() );
		e.printStackTrace();
	    }
	    catch (FinderException e) {
		finderFlag = true;
		System.out.println("FinderException: "+ e.getMessage() );
		e.printStackTrace();
	    }
	    catch (Exception e) {
		System.out.println("Exception: "+ e.getMessage() );
		e.printStackTrace();
	    }
	}
	
	// --- Prepare messages to accompany response. ---
	if (inputFlag) {
	    // The client input was OK - did transfer methods work?
	    if (transferFlag) {
		// Transfer methods worked
		if (request == 1)
		    messageLine1 = successtransfer;
		else
		    messageLine1 = successbalance;
	    }
	    else if (insufficientFlag) {
		// Not enough money in "from" account.
		messageLine1 = failureinsufficient;
		messageLine2 = amount + ": " + amountFloat + ", " +
		    from + ": " + fromKey + ", " +
		    to + ": " + toKey;
	    }
	    else if (finderFlag) {
		// One (or both) accounts not found.
		messageLine1 = failurefind;
		messageLine2 = from + ": " + fromKey + ", " +
		    to + ": " + toKey;
	    }
	    else {
		// Other reason for failure (might be non-existent account)
		messageLine1 = failureinternal;
	    }
	}
      else {
	  // Something was wrong with the client input
	  String separator = "";
	  if (!amountFlag) {
	      messageLine1 = failureinput;
	      messageLine2 = failureamount + " " + amountArray[0];
	      separator = ", ";
	  }
	  if (!fromAccountFlag) {
	      messageLine1 = failureinput;
	      messageLine2 = messageLine2 + separator + failureaccount +
		  " " + fromAccountArray[0];
	      separator = ", ";
	  }
	  if (!toAccountFlag) {
	      messageLine1 = failureinput;
	      messageLine2 = messageLine2 + separator + failureaccount +
		  " " + toAccountArray[0];
	  }
      }

	// --- Prepare HTML response. ---
	// HTML returned looks like initial HTML that invoked this servlet.
	// Message line says whether servlet was successful or not.
	res.setContentType("text/html");
	res.setHeader("Pragma", "no-cache");
	res.setHeader("Cache-control", "no-cache");
	PrintWriter out = res.getWriter();
	
	out.println("<html>");
	out.println("<head>");
	out.println("<title>" + transferTitle + "</title>");
	out.println("</head>");
	out.println("<body>");
	out.println("<h1 align=\"center\">" + transferTitle + "</h1>");
	out.println("<form name=\"form\" method=\"get\" action=" +
		    "\"/WebSphereSamples/servlet/WebSphereSamples.AccountAndTransfer.TransferFunds\">");
	out.println("<table border align=\"center\" width=\"500\">");
	out.println("<tr bgcolor=\"white\">");
	out.println("<th align=\"center\" width=\"33%\">" + amount + "</th>");
	out.println("<th align=\"center\" width=\"33%\">" + from + "</th>");
	out.println("<th align=\"center\" width=\"33%\">" + to + "</th>");
	out.println("</tr>");
	out.println("<tr bgcolor=\"#cccccc\">");
	out.println("<td align=\"center\"><input type=\"text\" name=\"amount\" " +
		    "size=\"20\" maxlength=\"10\"></td>");
	out.println("<td align=\"center\"><input type=\"text\" " +
		    "value=\"" + fromAccount + "\" name=\"fromaccount\" " +
		    "size=\"20\" maxlength=\"10\"></td>");
	out.println("<td align=\"center\"><input type=\"text\" " +
		    "value=\"" + toAccount + "\" name=\"toaccount\" " +
		    "size=\"20\" maxlength=\"10\"></td>");
	out.println("</tr>");
	out.println("<tr bgcolor=\"white\">");
	out.println("<th align=\"center\"></th>");
	out.println("<th align=\"center\">" + transferBalance + "</th>");
	out.println("<th align=\"center\">" + transferBalance + "</th>");
	out.println("</tr>");
	out.println("<tr bgcolor=\"#cccccc\">");
	out.println("<td align=\"center\"><input type=\"submit\" name=\"transfer\" " +
		    "value=\"" + transfer + "\"></td>");
	out.println("<td align=\"center\">" + fromBalance + "</td>");
	out.println("<td align=\"center\">" + toBalance + "</td>");
	out.println("</tr>");
	out.println("<tr bgcolor=\"#cccccc\">");
   out.println("<td align=\"center\"><input type=\"button\" value=\"" + clear + "\" " +
          "onClick=\"document.form.fromaccount.value='';document.form.toaccount.value='';document.form.amount.value=''\"> </td>");
	out.println("<td align=\"center\"><input type=\"submit\" name=\"fromgetbalance\" value=\"" +
		    get + "\"></td>");
	out.println("<td align=\"center\"><input type=\"submit\" name=\"togetbalance\" value=\"" +
		    get + "\"></td>");
	out.println("</tr>");
	out.println("</table>");
	out.println("</form>");
	out.println("<table align=\"center\">");
	out.println("<tr bgcolor=\"white\">");
	out.println("<td align=\"center\">");
	out.println(messageLine1);
	out.println("</td>");
	out.println("</tr>");
	out.println("<tr>");
	out.println("<td align=\"center\">");
	out.println(messageLine2);
	out.println("</td>");
	out.println("</tr>");
	out.println("</table>");
	out.println("</form>");
	out.println("</body>");
	out.println("</html>");
    }

}
