// 5648-C84, 5648-C83, (C) Copyright IBM Corporation, 1997, 2000
// All rights reserved. Licensed Materials Property of IBM
// Note to US Government users: Documentation related to restricted rights
// Use, duplication or disclosure is subject to restrictions set forth in GSA ADP Schedule with IBM Corp.
// This page may contain other proprietary notices and copyright information, the terms of which must be observed and followed.
//
// This program may be used, executed, copied, modified and distributed
// without royalty for the purpose of developing, using,
// marketing, or distributing.
//
// @(#) 1.3 com/java/src/com/ibm/ejb/cb/samples/hello/tier2/HelloBean.java, ejbdemo, bb98b, bb98b9838b 9/21/98 16:35:43 [9/22/98 16:39:13]


package com.ibm.ejb.cb.samples.hello.tier2;

import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.ejb.EJBException;
import java.rmi.RemoteException;
import java.util.ResourceBundle;
import java.util.ListResourceBundle;

/**
  The HelloBean class is a session Enterprise JavaBean (EJB) that
  returns a greeting.  Honors the methods declared in the Hello interface.
  <p>
  The deployment descriptor should mark this bean as a stateless session
  EJB.  The transaction attribute should be TX_NOT_SUPPORTED.
  See the <tt>EJBhello.xml</tt> file for full details.

  @see com.ibm.ejb.cb.samples.hello.tier2.Hello
  @see com.ibm.ejb.cb.samples.hello.tier2.HelloHome
  @see com.ibm.ejb.cb.samples.hello.tier2.HelloBeanResourceBundle
*/

public class HelloBean extends Object implements SessionBean {

  private static final String kIBMCopyright
      = "(c) Copyright IBM Corporation 1999";

  private static transient ListResourceBundle msgs = null;

  /**
    Creates an instance of a Hello bean.
  */
  public HelloBean ()
  {
    super();
  }

  /**
    This is not a part of SessionBean interface; it is defined implicitly
    through the "home" interface.  This method is called after creation.
  */
  public void ejbCreate ()   {
    // nothing needed to be done by me
  }

  /**
    The activate method is called when the instance is activated
    from its "passive" state.
    The instance should acquire any resource that it has released
    earlier in the ejbPassivate() method.

    @exception RemoteException Thrown if the remote method call fails.
  */
  public void ejbActivate () throws RemoteException
  {
    // nothing needed to be done by me
  }

  /**
    The passivate method is called before the instance enters the
    "passive" state.
    The instance should release any resources that that it
    can re-acquire later in the ejbActivate() method.

    @exception RemoteException Thrown if the remote method call fails.
  */
  public void ejbPassivate () throws RemoteException
  {
    // nothing needed to be done by me
  }

  /**
    A container invokes this method before it ends the life of
    the session object.
    This happens as a result of a client's invoking a remove
    operation, or when a container decides to terminate the session
    object after a timeout.

    @exception RemoteException Thrown if the remote method call fails.
  */
  public void ejbRemove () throws RemoteException
  {
    msgs = null;
  }

  /**
    Set the associated session context.
    The container calls this method after the instance creation.

    @param ctx An SessionContext interface for the instance.
    @exception RemoteException Thrown if the remote method call fails.
  */
  public void setSessionContext (SessionContext ctx) throws RemoteException
  {
    // we don't need the context, so we don't remember it
  }

  /**
    Returns the greeting.
    @return The greeting.
    @exception RemoteException Thrown if the remote method call fails.
  */
  public String getMessage () throws RemoteException
  {
    if ( msgs == null ) {
      msgs = (ListResourceBundle)
        ResourceBundle.getBundle("com.ibm.ejb.cb.samples.hello.tier2.HelloBeanResourceBundle");
    }
    return (String) msgs.handleGetObject( "greeting" );  // "Hello World!"
  }

}
