// 5648-C84, 5648-C83, (C) Copyright IBM Corporation, 1997, 2000
// All rights reserved. Licensed Materials Property of IBM
// Note to US Government users: Documentation related to restricted rights
// Use, duplication or disclosure is subject to restrictions set forth in GSA ADP Schedule with IBM Corp.
// This page may contain other proprietary notices and copyright information, the terms of which must be observed and followed.
//
// This program may be used, executed, copied, modified and distributed
// without royalty for the purpose of developing, using,
// marketing, or distributing.

package WebSphereSamples.HelloEJB;

// EJB imports
import com.ibm.ejb.cb.samples.hello.tier2.Hello;      // Hello EJB
import com.ibm.ejb.cb.samples.hello.tier2.HelloHome;  // Hello EJB's Home
import java.rmi.RemoteException;

// general Java
import java.util.*;
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

// JNDI-related
import javax.naming.InitialContext;
import WebSphereSamples.HelloEJB.HelloEJBServletResourceBundle;


public class HelloEJBServlet extends HttpServlet {

   // Copyright notice:
   private final static String kIBMCopyright =
   "(c) Copyright IBM Corporation 1999";

   // static data
   private static transient ListResourceBundle msgs = null;


   /**
     Drive the <b>Hello</b> Enterprise JavaBean.
   */
   public void doGet(HttpServletRequest req, HttpServletResponse res) throws IOException {

      PrintWriter out = res.getWriter();
	if ( msgs == null ) {
         msgs = (ListResourceBundle)
                ResourceBundle.getBundle("WebSphereSamples.HelloEJB.HelloEJBServletResourceBundle");
         System.out.println("****Trying to get HelloEJBServletResourceBundle");
      }

      try {
         // create ejb
         System.out.println( msgs.handleGetObject( "creating" ) );
         Hello hello = getHome().create();
         System.out.println("****creating hello");
         // call methods
         System.out.println( msgs.handleGetObject( "retrieving"));
         out.println( hello.getMessage() );

         // destroy ejb
         System.out.println( msgs.handleGetObject( "removing" ) );
         hello.remove();
         System.out.println( msgs.handleGetObject( "done") );
      }
      catch (RemoteException e) {
         out.println( msgs.handleGetObject( "remoteException") + e.getMessage() );
         e.printStackTrace();
      }
      catch (NullPointerException e) {
         if (getHome() == null)
            out.println( msgs.handleGetObject( "noHome" ) + e.getMessage() );
         else
            e.printStackTrace();
      }
      catch (Exception e) {
         out.println( msgs.handleGetObject( "generalException" ) + e.getMessage() );
         e.printStackTrace();
      }
   }

   /** Fake constructor.  Does nothing.  Sorry folks.
       Use the <i>main</i> method to actually drive the
       <b>Hello</b> Enterprise JavaBean.
       This constructor exists simply to keep <i>javadoc</i> happy. */
   public HelloEJBServlet () {
      super();              // see comment above
   }

   /** Helper method to get the EJB's Home. */
   private static HelloHome getHome () {

      String homeName = "HelloHome";

      try {

         InitialContext ctx = new InitialContext();
         System.out.println("****HelloHome found");
         return (HelloHome)ctx.lookup( homeName );
         
      }
      catch (javax.naming.NamingException e) {
         System.out.println("****HelloHome not found");
         e.printStackTrace();
         return null;
      }
   }

   /**
     Provides NLS support by retrieving the message from the resource bundle.
   */
  /* private static String getMsg ( String id ) {
      if ( msgs == null ) {
         msgs = (ListResourceBundle)
                ResourceBundle.getBundle("HelloEJBServletResourceBundle");
         System.out.println("****Trying to get HelloEJBServletResourceBundle");
      }
      return (String) msgs.handleGetObject( id );
   }*/

   /** Provides NLS support by retrieving the message from the resource bundle. */
   /*private static String getMsg ( String id, String arg1 ) {
      Object [] args = { arg1};
      return java.text.MessageFormat.format( getMsg( id ), args );
   }*/

}
