// 5648-C84, 5648-C83, (C) Copyright IBM Corporation, 1997, 2000
// All rights reserved. Licensed Materials Property of IBM
// Note to US Government users: Documentation related to restricted rights
// Use, duplication or disclosure is subject to restrictions set forth in GSA ADP Schedule with IBM Corp.
// This page may contain other proprietary notices and copyright information, the terms of which must be observed and followed.
//
// This program may be used, executed, copied, modified and distributed
// without royalty for the purpose of developing, using,
// marketing, or distributing.

package WebSphereSamples.Increment;

/**
 * Servlet to access the Increment Enterprise Java Bean.
 */

// General enterprise bean code.
import java.rmi.RemoteException;
import javax.rmi.PortableRemoteObject;
import javax.ejb.FinderException;
import javax.ejb.DuplicateKeyException;

// Enterprise bean code specific for this servlet.
import WebSphereSamples.Increment.IncrementHome;
import WebSphereSamples.Increment.IncrementKey;
import WebSphereSamples.Increment.Increment;

// Servlet related.
import javax.servlet.*;
import javax.servlet.http.*;

// JNDI (naming) for Context, InitialContext, NamingException
import javax.naming.*;  

// Miscellaneous:
import java.util.*;
import java.io.*;

public class VisitIncrementSite extends HttpServlet 
{
   // Variables 
   private IncrementHome incrementHome = null;
   private String primaryKey           = "count";
   private int count                   = 0;

   private String title    = "Visit the Increment Site";
   private String heading  = "Increment a Counter";
   private String subhead  = "Each time you visit the increment site, the counter is incremented by one.";
   private String number   = "Number of visits to Increment site:";
   private String again    = "Visit Again";
   private String factory  = null;
   private String provider = null;

   // Increment.properties property file located in same WebSphereSamples/Increment subdirectory
   // as the VisitIncrementSite servlet (which is in package WebSphereSamples.Increment),
   // hence the name "WebSphereSamples.Increment.Increment" to find it. 
   private static final String CONFIG_BUNDLE_NAME = "WebSphereSamples.Increment.Increment"; 

   public void init(ServletConfig config) throws ServletException
   {
      super.init(config);
      try
      {
         // Get information at runtime (from an external property file
         // identified by CONFIG_BUNDLE_NAME).
         PropertyResourceBundle configBundle = 
         (PropertyResourceBundle)PropertyResourceBundle.
         getBundle(CONFIG_BUNDLE_NAME);

         title    = configBundle.getString("title");
         heading  = configBundle.getString("heading"); 
         number   = configBundle.getString("number");  
         again    = configBundle.getString("again");   
         factory  = configBundle.getString("factory"); 
         provider = configBundle.getString("provider");
      }
      catch (Exception e)
      {
         System.out.println("Properties file exception: " + e.getMessage());
      }

      // Get home object for Increase enterprise bean access.
      Hashtable env = new Hashtable();

      env.put(Context.PROVIDER_URL, provider);
      env.put(Context.INITIAL_CONTEXT_FACTORY, factory);

      try
      {
         // Create the initial context.
         InitialContext ctx = new InitialContext(env);
         // Get the home object.
         Object homeObject = ctx.lookup("Increment");
         // Get the IncrementHome object.
         incrementHome = (IncrementHome)javax.rmi.PortableRemoteObject.narrow((org.omg.CORBA.Object) homeObject, IncrementHome.class);
      }
      // Determine cause of failure.
      catch (NamingException e)
      {
         // handle the exception, then throw a servlet exception to warn the servlet engine
         // and prevent execution of uninitialized servlets
         System.out.println("init NamingException: " + e.getMessage() );
         e.printStackTrace();
         throw new ServletException(e.getMessage());
      }
      catch (Exception e)
      {
         // handle the exception, then throw a servlet exception to warn the servlet engine
         // and prevent execution of uninitialized servlets
         System.out.println("init Exception: "+ e.getMessage() );
         e.printStackTrace();
         throw new ServletException(e.getMessage());
      }

      try
      {
         // get the primary key
         IncrementKey key = new IncrementKey(primaryKey);
         // create the home object - puts initial row in database if it is not already there.
         // results in one row, with "count" in primaeykey column, and 0 in count column.
         Increment myIncrement = incrementHome.create(key);
      }
      // Determine cause of failure.
      catch (DuplicateKeyException e)
      {
         // ignore this exception - code in try block puts in a "counter" row for the
         // first servlet access as a convenience, and from then on, the servlet uses
         // the "counter" row and does not need another one.
      }
      catch (RemoteException e)
      {
         System.out.println("RemoteException: " + e.getMessage() );
         e.printStackTrace();
         throw new ServletException(e.getMessage());
      }
      catch (Exception e)
      {
         System.out.println("Exception: "+ e.getMessage() );
         e.printStackTrace();
         throw new ServletException(e.getMessage());
      }
   }

   public void doGet(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException
   {
      // Question mark is visual indication on client of a failure.
      String countString = "?";

      try
      {
         // get the primary key
         IncrementKey key = new IncrementKey(primaryKey);
         // get the home object
         Increment myIncrement = incrementHome.findByPrimaryKey(key);
         // increment the count
         count = myIncrement.increment();
         countString = "" + count;
      }
      // Determine cause of failure.
      catch (RemoteException e)
      {
         System.out.println("RemoteException: " + e.getMessage() );
         e.printStackTrace();
      }
      catch (Exception e)
      {
         System.out.println("Exception: "+ e.getMessage() );
         e.printStackTrace();
      }

      // --- Prepare and send HTML response. ---
      res.setContentType("text/html");
      res.setHeader("Pragma", "no-cache");
      res.setHeader("Cache-control", "no-cache");
      PrintWriter out = res.getWriter();

      out.println("<html>");
      out.println("<head>");
      out.println("<title>" + title + "</title>");
      out.println("</head>");
      out.println("<body>");
      out.println("<h1 align=\"center\">" + heading + "</h1>");
      out.println("<form method=\"get\" action=\"/WebSphereSamples/servlet/WebSphereSamples.Increment.VisitIncrementSite\">");
      out.println("<p align=\"center\">" + subhead + "</p>");
      out.println("<table border align=\"center\">");
      out.println("<tr>");
      out.println("<td align=\"right\">" + number + " " + countString + "</td>");
      out.println("</tr>");
      out.println("<tr>");
      out.println("<td align=\"center\">");
      out.println("<input type=\"submit\" name=\"submit\" value=\"" + again + "\">");
      out.println("</tr>");
      out.println("</table>");
      out.println("</form>");
      out.println("</body>");
      out.println("</html>");
   }

}
