// 5648-C84, 5648-C83, (C) Copyright IBM Corporation, 1997, 2000
// All rights reserved. Licensed Materials Property of IBM
// Note to US Government users: Documentation related to restricted rights
// Use, duplication or disclosure is subject to restrictions set forth in GSA ADP Schedule with IBM Corp.
// This page may contain other proprietary notices and copyright information, the terms of which must be observed and followed.
//
// This program may be used, executed, copied, modified and distributed
// without royalty for the purpose of developing, using,
// marketing, or distributing.

package WebSphereSamples.YourCo.Timeout;

/**
* This is a Session Bean Class
*/
import java.rmi.RemoteException;
import java.security.Identity;
import java.util.Properties;
import javax.ejb.*;
import java.lang.*;
import javax.naming.*;
import java.util.*;
import WebSphereSamples.YourCo.Timeout.*;

public class AccessBean implements javax.ejb.SessionBean {
   private javax.ejb.SessionContext mySessionCtx = null;
   private InitialContext initialContext;
   private LeaveHome leaveHome;
   private HistoryHome historyHome;
   private AuditHome auditHome;

   // Property values to be read from the deployment descriptor.
   // See utility methods at end.
   private static final String CONTEXTFACTORY_PROP = "contextFactory";
   private static final String PROVIDERURL_PROP    = "providerURL";
   private static final String LEAVENAME_PROP      = "leaveName";
   private static final String HISTORYNAME_PROP    = "historyName";
   private static final String AUDITNAME_PROP      = "auditName";

/**
* ejbActivate method comment.
*/
   public void ejbActivate() throws java.rmi.RemoteException {
   }
/**
* ejbCreate method comment.
*/
   public void ejbCreate() {

      // --- get initial context
      try {
         java.util.Hashtable env = new java.util.Hashtable();
         env.put(Context.INITIAL_CONTEXT_FACTORY, getContextFactory());
         env.put(Context.PROVIDER_URL, getProviderURL());
         // Create the initial context.
         initialContext = new InitialContext(env);
      }
      // Determine cause of failure.
      catch (Exception e) {
         System.out.println("context Exception: " + e.getMessage());
         e.printStackTrace();
         System.exit(0);
      }

      // --- get home interface for Leave
      try {
         // Get the LeaveHome object.
         Object homeObject = initialContext.lookup(getLeaveName());
         leaveHome = (LeaveHome)javax.rmi.PortableRemoteObject.narrow(homeObject, WebSphereSamples.YourCo.Timeout.LeaveHome.class);
      }
      // Determine cause of failure.
      catch (Exception e) {
         System.out.println("Leave home interface Exception: " + e.getMessage());
         e.printStackTrace();
         System.exit(0);
      }

      // --- get home interface for History
      try {
         // Get the HistoryHome object.
         Object homeObject = initialContext.lookup(getHistoryName());
         historyHome = (HistoryHome)javax.rmi.PortableRemoteObject.narrow(homeObject, WebSphereSamples.YourCo.Timeout.HistoryHome.class);
      }
      // Determine cause of failure.
      catch (Exception e) {
         System.out.println("History home interface Exception: " + e.getMessage());
         e.printStackTrace();
         System.exit(0);
      }

      // --- get home interface for Audit
      try {
         // Get the AuditHome object.
         Object homeObject = initialContext.lookup(getAuditName());
         auditHome = (AuditHome)javax.rmi.PortableRemoteObject.narrow(homeObject, WebSphereSamples.YourCo.Timeout.AuditHome.class);
      }
      // Determine cause of failure.
      catch (Exception e) {
         System.out.println("Audit home interface Exception: " + e.getMessage());
         e.printStackTrace();
         System.exit(0);
      }
   }
/**
 * ejbPassivate method comment.
 */
   public void ejbPassivate() throws java.rmi.RemoteException {
   }
/**
 * ejbRemove method comment.
 */
   public void ejbRemove() throws java.rmi.RemoteException {
   }
/**
 * This method was created in VisualAge.
 * @return int
 * @param id int
 * @param type int
 * @exception java.rmi.RemoteException The exception description.
 * @exception javax.ejb.FinderException The exception description.
 */
   public int getBalance(int id, int type) throws java.rmi.RemoteException, FinderException {
      LeaveKey key = new LeaveKey(id, type);
      Leave leave = null;
      try {
         leave = leaveHome.findByPrimaryKey(key);
      }
      catch (FinderException e) {
         throw new FinderException("Leave id: " + id + ", type: " + type + " does not exist.");
      }
      return leave.getBalance();
   }
/**
 * This method was created in VisualAge.
 * @return Vector
 * @param id int
 * @exception java.rmi.RemoteException The exception description.
 * @exception javax.ejb.FinderException The exception description.
 */
   public Vector getHistory(int id) throws java.rmi.RemoteException, FinderException {
      try {
         Enumeration list = historyHome.findId(id);
         Vector result = new Vector();
         while (list.hasMoreElements()) {
            result.addElement(list.nextElement());
         }
         return result;
      }
      catch (FinderException e) {
         throw new FinderException("Leave id: " + id + " does not exist.");
      }
   }
/**
 * setSessionContext method comment.
 */
   public void setSessionContext(javax.ejb.SessionContext ctx) throws java.rmi.RemoteException {
      mySessionCtx = ctx;
   }
/**
 * This method was created in VisualAge.
 * @param id int
 * @param fromType int
 * @param toType int
 * @param amount int
 * @exception WebSphereSamples.YourCo.Timeout.InsufficientLeaveException The exception description.
 * @exception java.rmi.RemoteException The exception description.
 * @exception javax.ejb.FinderException The exception description.
 */
   public void transferDays(int id, int fromType, int toType, int amount) throws InsufficientLeaveException,
   java.rmi.RemoteException, FinderException, javax.ejb.CreateException, Exception {
      LeaveKey fromLeaveKey = new LeaveKey(id, fromType);
      LeaveKey toLeaveKey = new LeaveKey(id, toType);
      Leave fromLeave = null;
      Leave toLeave = null;
      History history = null;
      Audit audit = null;
      int auditNumber = 0;
      // --- get access to the "from" row of Leave enterprise bean
      try {
         fromLeave = leaveHome.findByPrimaryKey(fromLeaveKey);
      }
      catch (FinderException ex) {
         throw new FinderException("from id: " + id + ", type: " + fromType + " does not exist.");
      }
      // --- get access to the "to" row of Leave enterprise bean
      try {
         toLeave = leaveHome.findByPrimaryKey(toLeaveKey);
      }
      catch (FinderException ex) {
         throw new FinderException("to id: " + id + ", type: " + toType + " does not exist.");
      }
      // --- attempt to transfer hours and all that it involves
      try {
         // attempt to transfer hours within Leave enterprise bean
         toLeave.add(amount);
         fromLeave.subtract(amount);

         // get an auditNumber from Audit enterprise bean
         AuditKey key = new AuditKey("auditnumber");
         // if this is a first time run, the auditNumber row in the persistent table
         // will not exist - check for this and create the initial auditNumber of
         // 0 if this is the case.
         try {
            audit = auditHome.findByPrimaryKey(key);
         }
         catch (FinderException e1) {
            // no auditNumber row with key of "auditnumber" so create it
            try {
               audit = auditHome.create(key);
            }
            catch (Exception e2) {
               // cannot create the initial auditNumber row for some reason
               mySessionCtx.setRollbackOnly();
               throw e2;
            }
         }
         auditNumber = audit.incrementAuditNumber();

         // add information on transaction to History enterprise bean
         long dateLong = (new java.util.Date()).getTime();
         java.sql.Date theDate = new java.sql.Date(dateLong);
         java.sql.Time theTime = new java.sql.Time(dateLong);
         historyHome.create(auditNumber, id, fromType, toType, amount, theDate, theTime);
      }
      catch (InsufficientLeaveException ex) {
         mySessionCtx.setRollbackOnly();
         throw new InsufficientLeaveException("Insufficient leave, id: " + id + ", type: " + fromType);
      }
      catch (FinderException ex) {
         mySessionCtx.setRollbackOnly();
         throw new FinderException("no audit number available for key \"auditnumber\"");
      }
      catch (CreateException ex) {
         mySessionCtx.setRollbackOnly();
         throw new CreateException("could not create History entry, auditNumber: " + auditNumber + ", id: " + id + ", from type: " + fromType + ", to type: " + toType);
      }
      catch (Exception ex) {
         mySessionCtx.setRollbackOnly();
         throw ex;
      }

   }

   // Utility methods to get environmental variables from deployment descriptor.

   private String getContextFactory () throws RemoteException
   {
      // Get the contextFactory from the EJB properties.
      if (mySessionCtx == null)
         throw new RemoteException("noContext");

      if (mySessionCtx.getEnvironment() == null)
         throw new RemoteException("noEnvironment");

      String nf = mySessionCtx.getEnvironment().getProperty(CONTEXTFACTORY_PROP);
      if (nf == null)
         throw new RemoteException("envNotSet: " + CONTEXTFACTORY_PROP);

      return nf;
   }

   private String getProviderURL() throws RemoteException
   {
      // Get the provider URL property either from the EJB properties or, if
      // it isn't there use "iiop://", which causes a default to the local host.
      if (mySessionCtx == null)
         throw new RemoteException("noContext");

      if (mySessionCtx.getEnvironment() == null)
         throw new RemoteException("noEnvironment");

      String pr = mySessionCtx.getEnvironment().getProperty(PROVIDERURL_PROP);
      if (pr == null)
         pr = "iiop://";

      return pr;
   }

   private String getLeaveName() throws RemoteException
   {
      // Get the home name from the EJB properties.
      if (mySessionCtx == null)
         throw new RemoteException("noContext");

      if (mySessionCtx.getEnvironment() == null)
         throw new RemoteException("noEnvironment");

      String hn = mySessionCtx.getEnvironment().getProperty(LEAVENAME_PROP);
      if (hn == null)
         throw new RemoteException("envNotSet: " + LEAVENAME_PROP);

      return hn;
   }

   private String getHistoryName() throws RemoteException
   {
      // Get the home name from the EJB properties.
      if (mySessionCtx == null)
         throw new RemoteException("noContext");

      if (mySessionCtx.getEnvironment() == null)
         throw new RemoteException("noEnvironment");

      String hn = mySessionCtx.getEnvironment().getProperty(HISTORYNAME_PROP);
      if (hn == null)
         throw new RemoteException("envNotSet: " + HISTORYNAME_PROP);

      return hn;
   }

   private String getAuditName() throws RemoteException
   {
      // Get the home name from the EJB properties.
      if (mySessionCtx == null)
         throw new RemoteException("noContext");

      if (mySessionCtx.getEnvironment() == null)
         throw new RemoteException("noEnvironment");

      String hn = mySessionCtx.getEnvironment().getProperty(AUDITNAME_PROP);
      if (hn == null)
         throw new RemoteException("envNotSet: " + AUDITNAME_PROP);

      return hn;
   }

}
