// 5648-C84, 5648-C83, (C) Copyright IBM Corporation, 1997, 2000
// All rights reserved. Licensed Materials Property of IBM
// Note to US Government users: Documentation related to restricted rights
// Use, duplication or disclosure is subject to restrictions set forth in GSA ADP Schedule with IBM Corp.
// This page may contain other proprietary notices and copyright information, the terms of which must be observed and followed.
//
// This program may be used, executed, copied, modified and distributed
// without royalty for the purpose of developing, using,
// marketing, or distributing.

package WebSphereSamples.YourCo.Timeout;

// General enterprise bean code.
import java.rmi.RemoteException;
import javax.ejb.DuplicateKeyException;

// Enterprise bean code specific for this servlet.
import WebSphereSamples.YourCo.Timeout.LeaveHome;
import WebSphereSamples.YourCo.Timeout.LeaveKey;
import WebSphereSamples.YourCo.Timeout.Leave;

// Servlet related.
import javax.servlet.*;
import javax.servlet.http.*;

// JNDI (naming).
import javax.naming.*;  // for Context, InitialContext, NamingException

// Miscellaneous:
import java.util.*;
import java.io.*;

/**
 * Utility servlet to create an Leave entity bean.
 **/
public class LoadLeaveBeanTbl extends HttpServlet
{
   // Enterprise bean access.
   private LeaveHome leaveHome = null;
   private String factory      = null;
   private String providerURL  = null;
   private String leaveName    = null;

   public void init(ServletConfig config) throws ServletException
   {
      super.init(config);

      try
      {
         // YourCoEJB.properties property file located in same WebSphereSamples/YourCo 
         // subdirectory as the LoadLeaveBeanTbl servlet
         PropertyResourceBundle configBundle = 
         (PropertyResourceBundle)PropertyResourceBundle.getBundle("WebSphereSamples.YourCo.Timeout.YourCoEJB");

         factory     = configBundle.getString("factory"); 
         providerURL = configBundle.getString("providerURL");
         leaveName   = configBundle.getString("leaveName");
      }
      catch (Exception e)
      {
         System.out.println("Properties file exception: " + e.getMessage());
      }
   }

   public void doPost(HttpServletRequest req, HttpServletResponse res)
   throws ServletException, IOException
   {
      performTask(req, res);
   }

   public void doGet(HttpServletRequest req, HttpServletResponse res)
   throws ServletException, IOException
   {
      performTask(req, res);
   }

   private void performTask(HttpServletRequest req, HttpServletResponse res)
   throws ServletException, IOException
   {
      boolean success = false;

      res.setContentType("text/html");
      res.setHeader("Pragma", "no-cache");
      res.setHeader("Cache-control", "no-cache");
      PrintWriter out = res.getWriter();
      out.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\">");
      out.println("<html>");
      out.println("<head>");
      out.println("<LINK href=\"/theme/Master.css\" rel=\"stylesheet\" type=\"text/css\">");
      out.println("</head>");
      out.println("<body>");
      try
      {
         // Create the enterprise beans.
         success = load(30, 1, 4, out);
         success = load(30, 2, 5, out);
         success = load(30, 3, 6, out);
         success = load(120, 1, 3, out);
         success = load(120, 2, 7, out);
         success = load(120, 3, 5, out);
         success = load(190, 1, 5, out);
         success = load(190, 2, 2, out);
         success = load(190, 3, 8, out);
      }
      catch (Exception e)
      {
         success = false;
         // fail response
         out.println("<H2>The enterprise bean table EJB.LEAVEBEANTBL did NOT populate successfully.</H2>");
         out.println("<p>Make sure that the YourCo Leave enterprise bean has been successfully deployed and " + 
                     "that its server has been stopped and restarted.  Exception in performTask method.</p>");
         out.println("<p>A trace follows that may contain more information about the problem.</p>");
         e.printStackTrace(out);
      }
      if (success)
      {
         // success response
         out.println("<P><STRONG>Enterprise Beans Configuration is complete!</STRONG></P>");
         out.println("<P>After completing configuration, run a Sample chosen from the left frame, to learn how it works.");
      }
      out.println("</body>");
      out.println("</html>");
   }

   private boolean load(int id, int type, int balance, PrintWriter out) throws Exception
   {
      try
      {
         // Create the enterprise bean.
         if (leaveHome == null)
         {
            getLeaveHome(out);
         }
         leaveHome.create(id, type, balance);
      }
      // Determine cause of failure.
      catch (DuplicateKeyException e)
      {
         // do nothing - let existing row remain unchanged
      }
      catch (Exception e)
      {
         out.println("<H2>The enterprise bean table EJB.LEAVEBEANTBL did NOT populate successfully.</H2>");
         out.println("<p>Make sure that the YourCo Leave enterprise bean has been successfully deployed and " + 
                     "that its server has been stopped and restarted.  Exception in load method.</p>");
         out.println("<p>A trace follows that may contain more information about the problem.</p>");
         e.printStackTrace(out);
         out.println("</body>");
         out.println("</html>");
         throw e;
      }
      return true;
   }

   private void getLeaveHome(PrintWriter out) throws Exception
   {
      // Get home object for Leave enterprise bean access.
      Hashtable env = new Hashtable();
      env.put(Context.PROVIDER_URL, providerURL);
      env.put(Context.INITIAL_CONTEXT_FACTORY, factory);
      try
      {
         // Create the initial context.
         Context ctx = new InitialContext(env);
         // Get the home object.
         Object homeObject = ctx.lookup(leaveName);
         // Get the LeaveHome object.
         leaveHome = (LeaveHome)javax.rmi.PortableRemoteObject.narrow(homeObject, LeaveHome.class);
      }
      // Need to determine cause of failure.
      catch (Exception e)
      {
         out.println("<H2>The enterprise bean table EJB.LEAVEBEANTBL did NOT populate successfully.</H2>");
         out.println("<p>Make sure that the YourCo Leave enterprise bean has been successfully deployed and " + 
                     "that its server has been stopped and restarted.  Exception in getLeaveHome method.</p>");
         out.println("<p>A trace follows that may contain more information about the problem.</p>");
         e.printStackTrace(out);
         out.println("</body>");
         out.println("</html>");
         throw e;
      }
   }
}
