// 5648-C84, 5648-C83, (C) Copyright IBM Corporation, 1997, 2000
// All rights reserved. Licensed Materials Property of IBM
// Note to US Government users: Documentation related to restricted rights
// Use, duplication or disclosure is subject to restrictions set forth in GSA ADP Schedule with IBM Corp.
// This page may contain other proprietary notices and copyright information, the terms of which must be observed and followed.
//
// This program may be used, executed, copied, modified and distributed
// without royalty for the purpose of developing, using,
// marketing, or distributing.

package WebSphereSamples.YourCo.Timeout;

// General enterprise bean code.
import java.rmi.RemoteException;
import javax.ejb.FinderException;

// Enterprise bean code specific for this servlet.
import WebSphereSamples.YourCo.Timeout.AccessHome;
import WebSphereSamples.YourCo.Timeout.Access;
import WebSphereSamples.YourCo.Timeout.History;
import WebSphereSamples.YourCo.Timeout.Leave;

// necessary only while EJB and servlet are in different packages
import WebSphereSamples.YourCo.Timeout.InsufficientLeaveException;

// Imports
import com.ibm.servlet.*;
import com.ibm.webtools.runtime.*;
import javax.servlet.*;
import javax.servlet.http.*;

// JNDI (naming).
import javax.naming.*;  // for Context, InitialContext, NamingException

// Miscellaneous:
import java.util.*;
import java.io.*;

// Imports for beans used by this servlet
import WebSphereSamples.YourCo.Timeout.TimeoutBean;
import WebSphereSamples.YourCo.Login.SessionBean;

/**
 * Demonstration servlet showing a session enterprise bean accessing an
 * entity enterprise bean to get account balances or to access funds
 * between accounts.
 **/
public class TimeoutServlet extends com.ibm.webtools.runtime.StudioPervasiveServlet
{

   // Enterprise bean access.
   private AccessHome accessHome = null;


   // Process incoming requests
   public void performTask(HttpServletRequest req, HttpServletResponse res)
   {
      Access access = null;
      try
      {
         HttpSession session = req.getSession(true);
         WebSphereSamples.YourCo.Login.SessionBean sessionBean = null;
         sessionBean = (WebSphereSamples.YourCo.Login.SessionBean) session.getValue("sessionBean");

         if (sessionBean == null)
         {
            callPage("NotLoggedIn", req, res);
         }
         else
         {
            String idString = sessionBean.getEmpno();
            int idInt = Integer.parseInt(idString);

            int aBalance = 0;

            // instantiate the beans and store them so they can be accessed by the called page
            WebSphereSamples.YourCo.Timeout.TimeoutBean timeoutBean = null;
            timeoutBean = (WebSphereSamples.YourCo.Timeout.TimeoutBean) java.beans.Beans.instantiate(getClass().getClassLoader(), "WebSphereSamples.YourCo.Timeout.TimeoutBean");

            setRequestAttribute("timeoutBean", timeoutBean, req);

            // Create an Access object.
            if (accessHome == null)
            {
               accessHome = getAccessHome(req);
            }
            access = accessHome.create();

            String amountString = null;
            amountString = req.getParameter("amount");

            if (amountString != null)
            {
               String fromtypeString = req.getParameter("fromtype");
               String totypeString = req.getParameter("totype");

               int fromtypeInt = Integer.parseInt(fromtypeString);
               int totypeInt = Integer.parseInt(totypeString);
               int amountInt = Integer.parseInt(amountString);

               access.transferDays(idInt, fromtypeInt, totypeInt, amountInt);
            }

            for (int typeInt=1; typeInt<=3; typeInt++)
            {
               aBalance = access.getBalance(idInt, typeInt);
               timeoutBean.setBalance(typeInt-1, aBalance);
            }

            java.util.Enumeration list = null;
            list = (access.getHistory(idInt)).elements();    // convert Vector to Enumeration
            for (int index=0; list.hasMoreElements(); index++)
            {
               History temp =
               (History)javax.rmi.PortableRemoteObject.narrow(list.nextElement(), WebSphereSamples.YourCo.Timeout.History.class);
               timeoutBean.setFromType( index, temp.getFromType() );
               timeoutBean.setToType( index, temp.getToType() );
               timeoutBean.setAmount( index, Integer.toString(temp.getAmount()) );
               timeoutBean.setDate( index, temp.getDate().toString() );
               timeoutBean.setTime( index, temp.getTime().toString() );
               timeoutBean.setAuditNumber( index, Integer.toString(temp.getAuditNumber()) );
            }

            // Call the output page.
            String pageName = getPageNameFromRequest(req);
            callPage(pageName, req, res);
         }
      }

      // Determine approximate cause of failure.
      catch (FinderException e)
      {
         System.out.println("FinderException: " + e.getMessage() );
         e.printStackTrace();
         callPage("NotPopulated", req, res);
      }
      catch (RemoteException e)
      {
         System.out.println("RemoteException: " + e.getMessage() );
         e.printStackTrace();
         callPage("NotDeployed", req, res);
      }
      catch (InsufficientLeaveException e)
      {
         callPage("InsufficientLeave", req, res);
      }
      catch (PageNotFoundException e)
      {
         System.out.println("PageNotFoundException: " + e.getMessage() );
         e.printStackTrace();
      }
      catch (Exception e)
      {
         System.out.println("Exception: " + e.getMessage() );
         e.printStackTrace();
      }

      // Always attempt to free access session enterprise bean resource.
      finally
      {
         if (access != null)
         {
            try
            {
               access.remove();
            }
            catch (Exception e)
            {
               System.out.println("access.remove() exception: " +
                                  e.getMessage() );
            }
         }
      }
   }

   // Utility method
   protected AccessHome getAccessHome(HttpServletRequest request)
   {
      AccessHome aHome = null;

      try
      {
         // Get parameters from .servlet file
         String factory     = getParameter(request, "factory", true, true, false, null);
         String accessName  = getParameter(request, "accessName", true, true, false, null);
         String providerURL = getParameter(request, "providerURL", true, true, false, null);

         // Get home object for Access enterprise bean access.
         Hashtable env = new Hashtable();
         env.put(Context.PROVIDER_URL, providerURL);
         env.put(Context.INITIAL_CONTEXT_FACTORY, factory);

         // Create the initial context.
         Context ctx = new InitialContext(env);
         // Get the home object.
         Object homeObject = ctx.lookup(accessName);
         // Get the AccessHome object.
         aHome = (AccessHome)javax.rmi.PortableRemoteObject.narrow(homeObject, WebSphereSamples.YourCo.Timeout.AccessHome.class);
      }
      // Determine cause of failure
      catch (NamingException e)
      {
         System.out.println("getAccessHome NamingException: " + e.getMessage() );
         e.printStackTrace();
      }
      catch (Exception e)
      {
         System.out.println("getAccessHome Exception: "+ e.getMessage() );
         e.printStackTrace();
      }
      return aHome;
   }
}
