<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <xsl:output method="xml" omit-xml-declaration="no" indent="yes"/>
  <xsl:strip-space elements="*"/>
  <xsl:template match="/">
    <xsl:apply-templates select="web-app"/>
  </xsl:template>

  <xsl:template match="web-app">
    <webapp>
      <xsl:apply-templates select="display-name"/>
      <xsl:apply-templates select="web-app/description"/>
      <xsl:apply-templates select="error-page"/>
      <xsl:apply-templates select="session-config"/>
      <xsl:apply-templates select="mime-mapping"/>
      <xsl:apply-templates select="context-param"/>
      <xsl:apply-templates select="taglib"/>
      <xsl:apply-templates select="servlet"/>
      <xsl:apply-templates select="welcome-file-list"/>
      <servlet>
        <name>jsp11</name>
        <description>JSP 1.1 support servlet</description>
        <code>org.apache.jasper.runtime.JspServlet</code>
        <servlet-path>*.jsp</servlet-path>
        <servlet-path>*.jsv</servlet-path>
        <servlet-path>*.jsw</servlet-path>
        <autostart>true</autostart>
      </servlet>
      <servlet>
        <name>file</name>
        <description>File Serving servlet</description>
        <code>com.ibm.servlet.engine.webapp.SimpleFileServlet</code>
        <servlet-path>/</servlet-path>
        <autostart>true</autostart>
      </servlet>
      <servlet>
        <name>ErrorReporter</name>
        <description>Default error reporter servlet</description>
        <code>com.ibm.servlet.engine.webapp.DefaultErrorReporter</code>
        <servlet-path>/ErrorReporter</servlet-path>
        <autostart>true</autostart>
      </servlet>
    </webapp>
  </xsl:template>

  <xsl:template match="display-name">
    <name><xsl:value-of select="."/></name>
  </xsl:template>

  <xsl:template match="web-app/description">
    <description><xsl:value-of select="."/></description>
  </xsl:template>

  <xsl:template match="error-page">
    <error-page-j2ee>
      <xsl:apply-templates select="error-code"/>
      <xsl:apply-templates select="exception-type"/>
      <location><xsl:value-of select="location"/></location>
    </error-page-j2ee>
  </xsl:template>

  <xsl:template match="error-code">
      <error-code><xsl:value-of select="."/></error-code>
  </xsl:template>

  <xsl:template match="exception-type">
      <exception-type><xsl:value-of select="."/></exception-type>
  </xsl:template>

  <xsl:template match="session-config">
    <session-config>
      <session-timeout><xsl:value-of select="session-timeout"/></session-timeout>
    </session-config>
  </xsl:template>

  <xsl:template match="mime-mapping">
    <mime type="{mime-type}">
      <xsl:apply-templates select="//extension">
        <xsl:with-param name="mime-type-param">
          <xsl:value-of select="mime-type"/>
        </xsl:with-param>
      </xsl:apply-templates>  
    </mime>
  </xsl:template>

  <xsl:template match="//extension">
    <xsl:param name="mime-type-param"/>
    <xsl:variable name="extension-mime-type" select="parent::mime-type"/>
    <xsl:if test="$mime-type-param=$extension-mime-type">
      <ext><xsl:apply-templates select="."/></ext>
    </xsl:if>  
  </xsl:template>

  <xsl:template match="welcome-file-list">
    <welcome-file-list>
      <welcome-file><xsl:apply-templates select="welcome-file"/></welcome-file>
    </welcome-file-list>
  </xsl:template>

  <xsl:template match="welcome-file">
    <xsl:if test="position() > 1">,</xsl:if><xsl:value-of select="."/>
  </xsl:template>

  <xsl:template match="context-param">
    <init-parameter>
      <xsl:apply-templates select="param-name"/>
      <xsl:apply-templates select="param-value"/>
    </init-parameter>
  </xsl:template>

  <xsl:template match="taglib">
    <taglib-uri><xsl:value-of select="taglib-uri"/></taglib-uri>
    <taglib-location><xsl:value-of select="taglib-location"/></taglib-location>
  </xsl:template>

 <xsl:template match="servlet">
    <servlet>
      <xsl:apply-templates select="servlet-name"/>
      <xsl:apply-templates select="servlet/description"/>
      <xsl:apply-templates select="servlet-class"/>
      <xsl:apply-templates select="jsp-file"/>
      <xsl:apply-templates select="load-on-startup"/>
      <xsl:apply-templates select="init-param"/>
      <xsl:apply-templates select="//servlet-mapping">
        <xsl:with-param name="name-of-servlet">
          <xsl:value-of select="servlet-name"/>
        </xsl:with-param>
      </xsl:apply-templates>  
    </servlet>
  </xsl:template>

  <xsl:template match="servlet-name">
    <name><xsl:value-of select="."/></name>
  </xsl:template>

  <xsl:template match="servlet/description">
    <description><xsl:value-of select="."/></description>
  </xsl:template>

  <xsl:template match="servlet-class">
    <code><xsl:value-of select="."/></code>
  </xsl:template>

  <xsl:template match="jsp-file">
    <code><xsl:value-of select="."/></code>
  </xsl:template>

  <xsl:template match="load-on-startup">
    <autostart>true</autostart>
  </xsl:template>

  <xsl:template match="init-param">
    <init-parameter>
      <xsl:apply-templates select="param-name"/>
      <xsl:apply-templates select="param-value"/>
    </init-parameter>
  </xsl:template>

  <xsl:template match="param-name">
    <name><xsl:value-of select="."/></name>
  </xsl:template>

  <xsl:template match="param-value">
    <value><xsl:value-of select="."/></value>
  </xsl:template>

  <xsl:template match="//servlet-mapping">
    <xsl:param name="name-of-servlet"/>
    <xsl:variable name="sn" select="servlet-name"/>
    <xsl:if test="$name-of-servlet=$sn">
      <xsl:apply-templates select="url-pattern"/>
    </xsl:if>  
  </xsl:template>

  <xsl:template match="url-pattern">
    <servlet-path><xsl:value-of select="."/></servlet-path>
  </xsl:template>


</xsl:stylesheet>
